@@ -67,7 +67,7 @@ $target %dowhat readsweeps $max_processes $computype $calcprocedure %specularrat
 toil genstar solvestar integratebox filterbox__ cleanitem
 );
 
-$VERSION = '0.133';
+$VERSION = '0.135';
 $ABSTRACT = 'Sim::OPT is an optimization and parametric exploration program oriented to problem decomposition. It can be used with simulation programs receiving text files as input and emitting text files as output. It allows a free mix of sequential and parallel block coordinate searches.';
 
 #################################################################################
@@ -778,20 +778,20 @@ sub flattenbox
 
 sub integratebox
 {
-	my @arrelts = @{ $_[0] };
-	my %carrier = %{ $_[1] };
+	my @arrelts = @{ $_[0] }; say $tee "IN INTEGRATEBOX \@arrelts " . dump( @arrelts ) ;
+	my %carrier = %{ $_[1] }; say $tee "IN INTEGRATEBOX \%carrier " . dump( \%carrier ) ;
 	my $file = $_[2];
-	my @blockelts = @{ $_[3] };
+	my @blockelts = @{ $_[3] }; say $tee "IN INTEGRATEBOX \@blockelts @blockelts" ;
 	my @newbox;
 	foreach my $eltref ( @arrelts )
 	{
 		my @elts = @{ $eltref };
 		my $target = $elts[0]; #say $tee "IN INTEGRATEBOX Target $target" ;
-		my $righttarget = extractcase( $target, \%carrier, $file, \@blockelts ); #say $tee "IN INTEGRATEBOX Target $target" ;
+		my $righttarget = extractcase( $target, \%carrier, $file, \@blockelts ); say $tee "IN INTEGRATEBOX \$righttarget $righttarget" ;
 		my $lasttarget = $elts[3]; #say $tee "IN INTEGRATEBOX lasttarget $lasttarget" ;
 		my $pasttarget = extractcase( $lasttarget, \%carrier, $file, @blockelts ); #say $tee "IN INTEGRATEBOX pasttarget $pasttarget" ;; #say $tee "IN INTEGRATEBOX Target $target" ;
 		push (@newbox, [ $righttarget, $elts[1], $elts[2], $pasttarget, $elts[2] ] );
-	}
+	} say $tee "IN INTEGRATEBOX \@newbox " . dump( @newbox ) ;
 	return ( \@newbox );
 }
 
@@ -1240,38 +1240,38 @@ sub callblock # IT CALLS THE SEARCH ON BLOCKS.
 
 		%dirfiles = %{ solvestar( { dirfiles => \%dirfiles, dowhat => \%dowhat, varnumbers => \@varnumbers, blockelts => \@blockelts, carrier => \%carrier } ) };
 
-		my @newmiditers = @{ $dirfiles{starpositions} }; #say $tee "IN2 callblock \@newmiditers " . dump( @newmiditers );
+		my @newmiditers = @{ $dirfiles{starpositions} }; say $tee "IN2 callblock \@newmiditers " . dump( @newmiditers );
 		# IT CORRECTS THE PART OF @miditers OUTSIDE THE BLOCKS
 
 		$dirfiles{miditershold} = dclone( \@miditers ); say $tee "IN2 BEFORE callblock  \$dirfiles{miditershold} " . dump( $dirfiles{miditershold} );
 
-		sub checkmids
-		{
-			my ( $newmiditers_r, $miditershold ) = @_;
-			my @newmiditers = @{ $newmiditers_r };
-			my @oldmiditers = @{ $miditershold };
-			my $oldmids_r = $oldmiditers[0];
-			my %oldmids = %{ $oldmids_r };
-			my @news;
-			foreach my $mids_r ( @newmiditers )
-			{
-				my %mids = %{ $mids_r };
-				foreach my $key ( keys %mids )
-				{
-					if ( not ( $key ~~ @blockelts ) )
-					{
-						$mids{$key} = $oldmids{$key};
-					}
-				}
-				push( @news, \%mids );
-			}
-			return( \@news );
-		}
+		#sub checkmids
+		#{
+		#	my ( $newmiditers_r, $miditershold ) = @_;
+		#	my @newmiditers = @{ $newmiditers_r };
+		#	my @oldmiditers = @{ $miditershold };
+		#	my $oldmids_r = $oldmiditers[0];
+		#	my %oldmids = %{ $oldmids_r };
+		#	my @news;
+		#	foreach my $mids_r ( @newmiditers )
+		#	{
+		#		my %mids = %{ $mids_r };
+		#		foreach my $key ( keys %mids )
+		#		{
+		#			if ( not ( $key ~~ @blockelts ) )
+		#			{
+		#				$mids{$key} = $oldmids{$key};
+		#			}
+		#		}
+		#		push( @news, \%mids );
+		#	}
+		#	return( \@news );
+		#}
+		#
+		#@newmiditers = @{ checkmids( \@newmiditers, $dirfiles{miditershold} ) }; #say $tee "IN2 callblock POST-PROCESSED \@newmiditers " . dump( @newmiditers );
 
-		@newmiditers = @{ checkmids( \@newmiditers, $dirfiles{miditershold} ) }; #say $tee "IN2 callblock POST-PROCESSED \@newmiditers " . dump( @newmiditers );
-
-		@miditers = @{ dclone( \@newmiditers ) };
-	  @miditers = Sim::OPT::washn( @miditers ); #say $tee "IN2 AFTER callblock dcloned \@miditers " . dump( @miditers );
+		@miditers = @{ dclone( \@newmiditers ) }; say $tee "IN2 AFTER callblock dcloned \@miditers " . dump( @miditers );
+	  #@miditers = Sim::OPT::washn( @miditers ); say $tee "IN2 AFTER callblock dcloned \@miditers " . dump( @miditers );
 		#say $tee "IN2 callblock  \$dirfiles{starpositions} " . dump( $dirfiles{starpositions} );
 	}
 
@@ -1320,7 +1320,7 @@ sub deffiles # IT DEFINED THE FILES TO BE PROCESSED
   my $countblock = $d{countblock}; #say $tee "IN DEFFILES \$countblock : " . dump( $countblock );
 	my @sweeps = @{ $d{sweeps} }; #say $tee "IN DEFFILES \@sweeps : " . dump( @sweeps );
 	my @sourcesweeps = @{ $d{sourcesweeps} }; #say $tee "IN DEFFILES \@sourcesweeps : " . dump( @sourcesweeps );
-	my @miditers = @{ $d{miditers} }; #say $tee "IN DEFFILES \@miditers : " . dump( @miditers );
+	my @miditers = @{ $d{miditers} }; say $tee "IN DEFFILES \@miditers : " . dump( @miditers );
 	my @winneritems = @{ $d{winneritems} }; #say $tee "\@winneritems : " . dump( @winneritems );
 	my %dirfiles = %{ $d{dirfiles} }; #say $tee "IN deffiles \%dirfiles " . dump( %dirfiles );
 	#say $tee "IN DEFFILES \$dirfiles{starsign} " . dump( $dirfiles{starsign} );
@@ -1369,15 +1369,17 @@ sub deffiles # IT DEFINED THE FILES TO BE PROCESSED
 			foreach my $elt ( @basket )
 			{
 				my $root = $elt->[0]; #say $tee "IN TOIL \$root : " . dump( $root );
+				my @bag;
 				my $item;
 				my $cnstep = 1;
 				while ( $cnstep <= $maxvalue)
 				{	#say $tee "IN TOIL \$countblock : " . duvernummp( $countblock );
 					my $olditem = $root; #say $tee "IN TOIL \$olditem : " . dump( $olditem );
 					$item = "$root" . "$var" . "-" . "$cnstep" . "_" ;  #say $tee "IN TOIL \$item : " . dump( $item );
-					push ( @bucket, [ $item, $var, $cnstep, $olditem, $c ] ); #say $tee "IN TOIL \@bucket: " . dump( @bucket );
+					push ( @bag, [ $item, $var, $cnstep, $olditem, $c ] ); #say $tee "IN TOIL \@bucket: " . dump( @bucket );
 					$cnstep++; #say $tee "IN TOIL \$cnstep : " . dump( $cnstep );
 				}
+				push ( @bucket, @bag );
 				#push ( @bucket, [ $item, $var, $cnstep, $olditem, $c ] ); #say $tee "IN TOIL \@bucket: " . dump( @bucket );
 			}
 			@basket = ();
@@ -1413,30 +1415,31 @@ sub deffiles # IT DEFINED THE FILES TO BE PROCESSED
 	{
 		#say $tee "DOING BOX \$dirfiles{starsign} : " . dump( $dirfiles{starsign} );
 		@bux = @{ toil( \@blockelts, \%varnums, \@basket ) };
-		@bux = @{ $bux[-1] };
-	} #say $tee "IN DEFFILES2 \@bux: " . dump( @bux );
+		@bux = uniq( @{ $bux[-1] } );
+	} say $tee "IN DEFFILES2 \@bux: " . dump( @bux );
 	#say $tee "IN DEFFILES2 \$countblock : " . dump( $countblock );
 	#say $tee "IN DEFFILES \@blockelts : " . dump( @blockelts );
 
-	#my $flattened_r = flattenbox( \@bux ); 	say $tee "IN DEFFILES2 \$flattened_r: " . dump( $flattened_r );
+	#############my $flattened_r = flattenbox( \@bux ); 	say $tee "IN DEFFILES2 \$flattened_r: " . dump( $flattened_r );
 
 	my ( @finalbox );
 	if ( $dirfiles{starsign} eq "yes" )
 	{
 		my @bag;
-		foreach my $mids_r ( @miditers )
+		foreach my $midsurrs_r ( @miditers )
 		{
-			my %mids = %{ $mids_r }; #say $tee "IN DEFFILES \%mids" . dump( \%mids );
-	 		@bag = @{ integratebox( \@bux, \%mids, $file, \@blockelts ) };
+			say $tee "MIDITERS FOR MIDSURRS!: " . dump ( @miditers );
+			my %midsurrs = %{ $midsurrs_r }; #say $tee "IN DEFFILES \%midsurrs" . dump( \%midsurrs );
+	 		@bag = uniq( @{ integratebox( \@bux, \%midsurrs, $file, \@blockelts ) } );
 			push( @finalbox, @bag );
 		}
 	}
 	else
 	{
-		@finalbox = @{ integratebox( \@bux, \%carrier, $file, \@blockelts ) };
+		@finalbox = uniq( @{ integratebox( \@bux, \%carrier, $file, \@blockelts ) } );
 	}
 
-	@finalbox = sort { $a->[0] <=> $b->[0] } @finalbox;
+	@finalbox = sort { $a->[0] <=> $b->[0] } @finalbox; say $tee "In DEFFILES5 \@finalbox!: " . dump ( @finalbox );
 
 
 	setlaunch( {
@@ -1648,6 +1651,7 @@ sub genstar
 		}
 		push( @starpositions, @genpoints );
 	}
+	say $tee "IN genstar STARPOSITIONS BEFORE: " . dump( @starpositions );
 
 	my @bag;
 	if ( scalar( @blockelts > 0) )
@@ -1666,6 +1670,7 @@ sub genstar
 		}
 		@starpositions = @bag;
 	}
+	say $tee "IN genstar STARPOSITIONS AFTER: " . dump( @starpositions );
 
 	return( \@starpositions );
 }
