@@ -190,16 +190,16 @@ sub adjustmode
 
 sub preparearr
 {
-  my ( $lines_ref ) = @_;
+  my @lines = @_;
   my $optformat = "yes";
-  my @lines = @{ $lines_ref };
+  my @arr;
   if ( $lines[1] =~ /_/ )
   {
     foreach my $line ( @lines )
     {
       chomp( $line );
-      my @row = split( /,/ , $line );
-      @pars = split( /_/ , $row[0] );
+      my @row = split( /,/ , $line ); say $tee "IN PREPAREARR \@row " . dump( @row );
+      @pars = split( /_/ , $row[0] ); say $tee "IN PREPAREARR \@pars " . dump( @pars );
       if ( $row[1] eq undef )
       {
         push ( @arr, [ $row[0], [ @pars ] ] );
@@ -246,7 +246,7 @@ sub preparearr
         push ( @arr, [ $header, [ @pars ], $row[-1], 1 ] );
       }
     }
-  } #say $tee "ARR: " . dump( @arr );
+  } say $tee "ARR: " . dump( @arr );
   return( \@arr, $optformat );
 }
 
@@ -499,23 +499,23 @@ sub calcdistgrad
 {
   my ( $el1, $elt1, $factlevels_ref, $minreq, $maxdist, $el3, $elt3, $condweight, $el, $elt ) = @_; #say $tee "777c\$el1: " . dump( $el1 ); say $tee "777c\$elt1: " . dump( $elt1 ); say $tee "777c\$minreq: " . dump( $minreq );
   my %factlevels = %{ $factlevels_ref };
-  #say $tee "IN CALCDISTGRAD \$el1: " . dump( $el1 );
-  #say $tee "IN CALCDISTGRAD \$elt1: " . dump( $elt1 );
-  my @diff1 = diff( \@{ $el1 }, \@{ $elt1 } ); #say $tee "IN CALCDISTGRAD \@diff1: " . dump( @diff1 );
+  say $tee "IN CALCDISTGRAD \$el1: " . dump( $el1 );
+  say $tee "IN CALCDISTGRAD \$elt1: " . dump( $elt1 );
+  my @diff1 = diff( \@{ $el1 }, \@{ $elt1 } ); say $tee "IN CALCDISTGRAD \@diff1: " . dump( @diff1 );
   my $dist;
   my ( @factbag, @levbag, @stepbag, @da1, @da1par, @da2, @da2par, @diff2 );
   #say $tee "777\scalar( \@diff1 ): " . dump( scalar( @diff1 ) ); say $tee "777\$minreq: " . dump( $minreq );
   #say $tee "\$minreq->[0]: " . dump( $minreq->[0] );
   if ( ( scalar( @diff1 ) > 0 ) and ( scalar( @diff1 ) <= $minreq->[0] ) )
   {
-    @diff2 = diff( \@{ $elt1 } , \@{ $el1 } ); #say $tee "IN CALCDISTGRAD \@diff2: " . dump( @diff2 );
+    @diff2 = diff( \@{ $elt1 } , \@{ $el1 } ); say $tee "IN CALCDISTGRAD \@diff2: " . dump( @diff2 );
 
-    my %h1 = map { split( /-/ , $_ ) } @diff1; #say $tee "IN CALCDISTGRAD \%h1: " . dump( \%h1 );
-    @da1 = keys %h1; #say $tee "IN CALCDISTGRAD \@da1: " . dump( @da1 );
-    @da1par = values %h1; #say $tee "IN CALCDISTGRAD \@da1par: " . dump( @da1par );
-    my %h2 = map { split( /-/ , $_ ) } @diff2; #say $tee "IN CALCDISTGRAD \%h2: " . dump( \%h2 );
-    @da2 = keys %h2; #say $tee "IN CALCDISTGRAD \@da2: " . dump( @da2 );
-    @da2par = values %h2; #say $tee "IN CALCDISTGRAD \@da2par: " . dump( @da2par );
+    my %h1 = map { split( /-/ , $_ ) } @diff1; say $tee "IN CALCDISTGRAD \%h1: " . dump( \%h1 );
+    @da1 = keys %h1; say $tee "IN CALCDISTGRAD \@da1: " . dump( @da1 );
+    @da1par = values %h1; say $tee "IN CALCDISTGRAD \@da1par: " . dump( @da1par );
+    my %h2 = map { split( /-/ , $_ ) } @diff2; say $tee "IN CALCDISTGRAD \%h2: " . dump( \%h2 );
+    @da2 = keys %h2; say $tee "IN CALCDISTGRAD \@da2: " . dump( @da2 );
+    @da2par = values %h2; say $tee "IN CALCDISTGRAD \@da2par: " . dump( @da2par );
 
     my $c = 0;
     foreach my $lev1 ( @da1par )
@@ -528,7 +528,7 @@ sub calcdistgrad
         if ( $da1 == $da2 )
         {
           my $fact = $da1;
-          my $diffpar = abs( $lev1 - $lev2 ); #say $tee "22 \$diffpar: " . dump( $diffpar );
+          my $diffpar = abs( $lev1 - $lev2 ); say $tee "22 \$diffpar: " . dump( $diffpar );
           if ( ( $diffpar <= $minreq->[1] ) and ( $fact ne "" ) and ( $diffpar ne "" ) and ( $factlevels{stepsizes}{$fact} ne "" ) )
           {
             push ( @factbag, $fact );
@@ -548,8 +548,8 @@ sub calcdistgrad
 
   unless ( ( scalar( @levbag ) == 0 ) or ( scalar( @factbag ) == 0 ) or ( scalar( @stepbag ) == 0 ) )
   {
-    my $ordist = scalar( @levbag ); #say $tee "IN CALCDISTGRAD \$ordist: " . dump( $ordist );
-    my $rawdist = pythagoras( \@factbag, \@levbag, \@stepbag ); #say $tee "IN CALCDISTGRAD \$rawdist: " . dump( $rawdist );
+    my $ordist = scalar( @levbag ); say $tee "IN CALCDISTGRAD \$ordist: " . dump( $ordist );
+    my $rawdist = pythagoras( \@factbag, \@levbag, \@stepbag ); say $tee "IN CALCDISTGRAD \$rawdist: " . dump( $rawdist );
     my ( $dist, $strength );
 
     if ( $maxdist ne "" )
@@ -581,8 +581,8 @@ sub calcmaxdist
       my %hash = %{ $hash_ref }; #say $tee "00\%hash: " . dump( %hash );
       push ( @rawdists, $hash{rawdist} );
     }
-  }  #say $tee "00\@rawdists: " . dump( @rawdists );
-  my $maxdist = max( @rawdists ); #say $tee "00\$maxdist: " . dump( $maxdist );
+  }  say $tee "00\@rawdists: " . dump( @rawdists );
+  my $maxdist = max( @rawdists ); say $tee "00\$maxdist: " . dump( $maxdist );
   return( $maxdist );
 }
 
@@ -593,11 +593,15 @@ sub wei
     $factlevels_ref, $minreq_forgrad, $minreq_forinclusion, $minreq_forcalc, $minreq_formerge, $maxdist ) = @_;
   my @arr = @{ $arr_ref };
   #say $tee "ARR VERY_BEFORE: " . dump(@arr);
-  my %factlevels = %{ $factlevels_ref };  #say $tee "AND \%factlevels: " . dump( %factlevels );
+  my %factlevels = %{ $factlevels_ref };  say $tee "AND \%factlevels: " . dump( %factlevels );
   # my ( %magic, %wand, %spell, %bank );
 
+
   sub fillbank
   { #say $tee "NOW IN FILLBANK.";
+    my ( $arr_r, $minimumcertain, $minreq_forgrad, $maxdist, $condweight, $factlevels_r ) = @_;
+    my @arr = @{ $arr_r };
+    my %factlevels = %{ $factlevels_r };
     my %bank;
     foreach my $el ( @arr )
     { #say $tee "SO IN FIRST ARR CHECK" ;  say $tee "\$el->[1]: " . dump( $el->[1] ); #say $tee "EL: " . dump( $el );
@@ -610,7 +614,7 @@ sub wei
           { #say $tee "NOW CHECKING .";
 
             my ( $res_ref ) = calcdistgrad( $el->[1], $elt->[1], \%factlevels, $minreq_forgrad, $maxdist, $el->[3], $elt->[3], $condweight, $el, $elt );
-            #say $tee "\OBTAINED DIST \$res_ref: " . dump( $res_ref );
+            say $tee "\OBTAINED DIST \$res_ref: " . dump( $res_ref );
 
             my %d;
             my ( @diff1, @diff2, @da1, @da2, @da1par, @da2par );
@@ -618,8 +622,8 @@ sub wei
 
             unless ( !keys %{ $res_ref } )
             {
-              %d = %{ $res_ref }; #say $tee "FOUND \%d: " . dump( \%d );
-              @diff1 = @{$d{diff1}}; #say $tee "FOUND \@diff1: " . dump( @diff1 );
+              %d = %{ $res_ref }; say $tee "FOUND \%d: " . dump( \%d );
+              my @diff1 = @{$d{diff1}}; #say $tee "FOUND \@diff1: " . dump( @diff1 );
               @diff2 = @{$d{diff2}}; #say $tee "FOUND \@diff2: " . dump( @diff2 );
               @da1 = @{$d{da1}}; #say $tee "FOUND \@da1: " . dump( @da1 );
               @da2 = @{$d{da2}}; #say $tee "FOUND \@da2: " . dump( @da2 );
@@ -658,7 +662,7 @@ sub wei
                       my @sorted = sort( $d11, $d21 );
                       my $orderedpair = join( "-", @sorted );
                       my $trio = join( "-", $d10, $pair );
-                      my $orderedtrio = join( "-", $d10, $orderedpair );
+                      my $orderedtrio = join( "-", $d10, $orderedpair ); say $tee "\$orderedtrio: " . dump( $orderedtrio );
 
                       unless ( $trio eq "" )
                       {
@@ -723,34 +727,37 @@ sub wei
   }
 
 
-  my $bank_ref = fillbank;
-  my %bank =  %{ $bank_ref }; #say $tee "\%bank: " . dump( %bank );
+  my %bank = %{ fillbank( \@arr, $minimumcertain, $minreq_forgrad, $maxdist, $condweight, \%factlevels ) };
 
   sub clean
   {
-    my %bank = @_;
+    %bank = @_;
     foreach my $trio ( keys ( %bank ) )
     {
       my ( @grads, @ordists, @dists, @strengths );
       unless( ( $bank{$trio}{grad} eq "" ) or ( $bank{$trio}{ordists} eq "" )
         or ( $bank{$trio}{dists} eq "" ) or ( $bank{$trio}{strengths} eq "" ) )
       {
-        push ( @grads,$bank{$trio}{grad} );
-        push ( @ordists,$bank{$trio}{ordists} );
-        push ( @dists,$bank{$trio}{dists} );
-        push ( @strengths,$bank{$trio}{strengths} );
+        push ( @grads, $bank{$trio}{grad} );
+        push ( @ordists, $bank{$trio}{ordists} );
+        push ( @dists, $bank{$trio}{dists} );
+        push ( @strengths, $bank{$trio}{strengths} );
         $bank{$trio}{grad} = [ @grads ];
         $bank{$trio}{ordists} = [ @ordists ];
         $bank{$trio}{dists} = [ @dists ];
         $bank{$trio}{strengths} = [ @strengths ];
       }
     }
+    return( %bank );
   }
-  my %bank = clean( %bank );
-  my %bank =  %{ $bank_ref }; #say $tee "CLEANED \%bank: " . dump( %bank );
+  %bank = clean( %bank );
+  #my %bank =  %{ $bank_ref }; say $tee "CLEANED \%bank: " . dump( %bank );
 
   sub cyclearr
-  { #say $tee "NTH ARR: " . dump( @arr );
+  {
+    my ( $arr_r, $minreq_forinclusion, $minreq_forgrad ) = @_;
+    my @arr = @{ $arr_r };
+    say $tee "IN cyclearr ARR: " . dump( @arr );
     my %wand;
     my $coun = 0;
     foreach my $el ( @arr )
@@ -907,8 +914,8 @@ sub wei
     return( \%wand );
   }
 
-  my $wand_ref = cyclearr;
-  my %wand = %{ $wand_ref }; #say $tee "\%wand OUT: " . dump( %wand );
+  my $wand_ref = cyclearr( \@arr, $minreq_forinclusion, $minreq_forgrad );
+  my %wand = %{ $wand_ref }; say $tee "\%wand OUT: " . dump( %wand );
 
 
   my @limb0;
@@ -922,7 +929,7 @@ sub wei
       push ( @limb0, [ $wand{$ke}{name}, $wand{$ke}{bulk}, $soughtval, $totstrength ] ); #say $tee "\$avg: $avg"; say $tee "\${ \$magic{\$ke}{\$dee} }->[1] : ${ $magic{$ke}{$dee} }->[1] ";
     }
   }
-  #say $tee "LIMBO_WEI: " . dump( @limbo_wei );
+  say $tee "LIMBO_WEI: " . dump( @limbo_wei );
   return( @limb0 )
 } ##### END SUB wei
 
@@ -1404,29 +1411,29 @@ sub interlinear
   #require $confile; ############## FIX THIS!
 
   if ( $sourcef ne "" ){ $sourcefile = $sourcef; } #say $tee "CHECK5 \$sourcefile: " . dump( $sourcefile );
-  if ( $metafile ne "" ){ $newfile = $metafile; } say $tee "CHECK5 \$newfile: " . dump( $newfile );
+  if ( $metafile ne "" ){ $newfile = $metafile; } #say $tee "CHECK5 \$newfile: " . dump( $newfile );
 
   my @blockelts;
   if ( $blockelts_r ne "" ){ @blockelts = @{ $blockelts_r }; } #say $tee "CHECK5 \@blockelts: " . dump( @blockelts );
 
   my @mode = adjustmode( $maxloops, \@mode );
   open( SOURCEFILE, "$sourcefile" ) or die;
-  my @lines = <SOURCEFILE>;
+  my @lines = <SOURCEFILE>; say $tee "REALLY \@lines: " . dump( @lines );
   close SOURCEFILE;
 
   say $tee "Preparing the dataseries.";
   my $aarr_ref;
-  ( $aarr_ref, $optformat ) = preparearr( \@lines );
+  ( $aarr_ref, $optformat ) = preparearr( @lines );
 
-  my @aarr = @{ $aarr_ref }; #say $tee "REALLY \@aarr: " . dump( @aarr );
+  my @aarr = @{ $aarr_ref }; say $tee "REALLY \@aarr: " . dump( @aarr );
 
   say $tee "Checking factors and levels.";
-  my %factlevels = %{ prepfactlev( \@aarr ) }; #say $tee "REALLY \%factlevels: " . dump( \%factlevels );
+  my %factlevels = %{ prepfactlev( \@aarr ) }; say $tee "IN INTERLINEAR REALLY \%factlevels: " . dump( \%factlevels );
 
   my ( $factlev_ref ) = tellstepsize( \%factlevels );
-  my %factlev = %{ $factlev_ref }; #say $tee "REALLY \%factlev: " . dump( %factlev );
+  my %factlev = %{ $factlev_ref }; say $tee "REALLY \%factlev: " . dump( %factlev );
 
-  my $maxdist = calcmaxdist( \@aarr, \%factlev ); #say $tee "001\$maxdist: " . dump( $maxdist );
+  my $maxdist = calcmaxdist( \@aarr, \%factlev ); say $tee "001\$maxdist: " . dump( $maxdist );
 
   my $count = 0;
   while ( $count < $maxloops )
@@ -1441,7 +1448,7 @@ sub interlinear
       @arr = @arr2;
     }
 
-    say $tee "COUNT: " . dump( $count + 1 );
+    #say $tee "COUNT: " . dump( $count + 1 );
     my $mode__ = $mode[$count] ;
 
     my ( @limbo_wei, @limbo_purelin, @limbo_near, @limbo, %bank, %wand );
@@ -1520,7 +1527,7 @@ sub interlinear
     #  @limbo = mixlimbo( \@limbo_prov, \@limbo_near, $presence, $linearprecedence, \@weights );
     #}
 
-    say $tee "OBTAINED LIMBO: " . dump( @limbo );
+    #say $tee "OBTAINED LIMBO: " . dump( @limbo );
     say $tee "MIXING THE ARRAY UPDATES " . ( $count + 1 ) . " for $sourcefile";
     say $tee "THERE ARE " . scalar( @limbo ) . " ITEMS COMING OUT FROM THIS MIX " . ( $count + 1 );
 
